@@ -29,13 +29,13 @@ SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 __all__ = ['TestResult', 'TestCase', 'TestSuite',
            'TextTestRunner', 'TestLoader', 'FunctionTestCase', 'main',
            'defaultTestLoader', 'SkipTest', 'skip', 'skipIf', 'skipUnless',
-           'expectedFailure', 'TextTestResult', '__version__']
+           'expectedFailure', 'TextTestResult', 'installHandler', 
+           'registerResult', 'removeResult', '__version__']
 
-__version__ = '0.2.0'
+__version__ = '0.3.0'
 
 # Expose obsolete functions for backwards compatibility
 __all__.extend(['getTestCaseNames', 'makeSuite', 'findTestCases'])
-__all__.append('_TextTestResult')
 
 
 from unittest2.result import TestResult
@@ -43,13 +43,16 @@ from unittest2.case import (
     TestCase, FunctionTestCase, SkipTest, skip, skipIf,
     skipUnless, expectedFailure
 )
-from unittest2.suite import TestSuite
+from unittest2.suite import BaseTestSuite, TestSuite
 from unittest2.loader import (
     TestLoader, defaultTestLoader, makeSuite, getTestCaseNames,
     findTestCases
 )
 from unittest2.main import TestProgram, main
 from unittest2.runner import TextTestRunner, TextTestResult
+from unittest2.signals import installHandler, registerResult, removeResult
 
 # deprecated
-_TextTestResult = TextTestResult
\ No newline at end of file
+_TextTestResult = TextTestResult
+
+__unittest = True
\ No newline at end of file
