@@ -5,6 +5,9 @@ import time
 import unittest
 
 from unittest2 import result
+from unittest2.signals import registerResult
+
+__unittest = True
 
 
 class _WritelnDecorator(object):
@@ -126,10 +129,11 @@ class TextTestRunner(unittest.TextTestRunner):
     resultclass = TextTestResult
 
     def __init__(self, stream=sys.stderr, descriptions=True, verbosity=1,
-                 resultclass=None):
+                 failfast=False, resultclass=None):
         self.stream = _WritelnDecorator(stream)
         self.descriptions = descriptions
         self.verbosity = verbosity
+        self.failfast = failfast
         if resultclass is not None:
             self.resultclass = resultclass
 
@@ -139,6 +143,9 @@ class TextTestRunner(unittest.TextTestRunner):
     def run(self, test):
         "Run the given test case or test suite."
         result = self._makeResult()
+        result.failfast = self.failfast
+        registerResult(result)
+            
         startTime = time.time()
         startTestRun = getattr(result, 'startTestRun', None)
         if startTestRun is not None:
